"""
    Created on 2012-01-25
    @author: jldupont
"""

class SDB_Connect(Exception): pass

class SDB_CreateDomain(Exception): pass

class SDB_CreateItem(Exception): pass

class SDB_Access(Exception): pass
